<?php

/**
 * @copyright   Copyright (C) 2006 - 2018 Michael Richey. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */
defined('_JEXEC') or die;
jimport('joomla.filesystem.file');

class GeoIPHelper {

    protected static $instance;
    private $params;
    private $cache = array(
        'cc' => array(),
        'abbr' => array(),
        'name' => array()
    );

    final public static function getInstance($params = false) {
        if (!isset(static::$instance)) {
            static::$instance = new self;
            $tmp_path = JFactory::getConfig()->get('tmp_path');
            if (!JFile::exists($tmp_path . '/geoiploc.php')) {
                // sanity check - download the update if it doesn't exist
                self::update($tmp_path);
            } else {
                if ($params && !$params->get('cli', false)) {
                    // this is super bad for user experience, look into scheduling the CLI
                    $cache = JFactory::getCache('plg_system_geoip', '');
                    $cache->setCaching(1);
                    $cache->setLifeTime(1440 * 60);
                    $lucache = $cache->get('lastupdate', '') ?: 0;
                    $lastupdate = new DateTime();
                    $lastupdate->setTimestamp($lucache);
                    $now = new DateTime();
                    if ($now->diff($lastupdate)->days >= 1) {
                        self::update($tmp_path, $cache, $params, $now);
                    }
                }
            }
            require_once $tmp_path . '/geoiploc.php';
        }
        return static::$instance;
    }

    public function all($ip,$cache = true) {
        $cc = $this->lookupCC($ip,$cache);
        $abbr = $this->lookupAbbr($ip,$cache);
        $name = $this->lookupName($ip,$cache);
        return array('cc'=>$cc,'abbr'=>$abbr,'name'=>$name);
    }
    
    private function store($ip,$type,$data) {
        $this->cache[$type][$ip] = $data;
        return $this->cache[$type][$ip];
    }

    public function lookupCC($ip,$cache = true) {
        return $this->lookup($ip,$cache,'cc');
    }

    public function lookupAbbr($ip,$cache = true) {
        return $this->lookup($ip,$cache,'abbr');
    }

    public function lookupName($ip,$cache = true) {
        return $this->lookup($ip,$cache,'name');
    }
    
    private function lookup($ip,$cache = true,$type) {
        $ptype = array(
            'cc'=>'code',
            'abbr'=>'AbBr',
            'name'=>' NamE '
        );
        if($cache) {
            if (!isset($this->cache[$type][$ip])) {
                return $this->store($ip,$type,getCountryFromIP($ip,$ptype[$type]));
            }
            return $this->cache[$type][$ip];
        }
        return isset($this->cache[$type][$ip])?$this->cache['cc'][$ip]:getCountryFromIP($ip,$ptype[$type]);        
    }

    public static function update($tmp_path, $cache = null, $params = false, $now = false) {
        $zip = self::download();
        if (!JFile::write($tmp_path . '/geoip.zip', $zip)) {
            throw new Exception("Failed to store update");
        }
        unset($zip);
        $za = new ZipArchive;
        if ($za->open($tmp_path . '/geoip.zip')) {
            $za->extractTo($tmp_path);
            $za->close();
        } else {
            throw new Exception("Failed to open update");
        }
        if ($params && !$params->get('cli', false)) {
            $cache->store($now->format('U'), 'lastupdate', '');
        }
    }
    
    private static function download() {
        $url = "https://cdn.richeyweb.com/updates/geoip.zip";
        $http = JHttpFactory::getHttp();
        $response = $http->get($url);
        // Generic check on valid HTTP code
        if ($response->code > 299) {
            throw new Exception("Failed to download update");
        }
        return $response->body;
    }

}
